% ------------------------------------------------------------------
% Code to simulate logit data and estimate nonparametrically
% Plots estimates of own and cross-price elasticities 
% Written by James Brand
% ------------------------------------------------------------------
clear
J=2; % # products
T =2000; 
beta = -0.4; % price coefficient
varxi = 0.15; % standard deviation of xi (misnomer)

S = 200;
esep_own = zeros(S,10); 
esep_cross = zeros(S,10);
esep_own_dist = zeros(S,T);
G = 10; % size of grid on which to evaluate price elasticities

for si = 1:1:S
% ------------------------------------------------------
% Simulation
% ------------------------------------------------------
[s, pt, zt, xi] = simulate_logit(J,T, beta, varxi);
if si ==1
    p_points = linspace(prctile(pt(:,1),25),prctile(pt(:,1),75),G)';
end

% ------------------------------------------------------
% Set options for estimation and elasticity calculation
% ------------------------------------------------------
    included = ones(J);         % Matrix of ones and zeros, specifying which products are substitutes. 
                                    % Each row corresponds to an inverse demand function. 
                                    % Set element (j,k) to 1 if k is a substitute for j
                                    % Mostly helpful if you're incorporating model selection prior
                                    % to estimation. Otherwise set to ones(J). 
                                    
    bernO = 2*ones(J,1);        % Order of Bernstein Polynomial
    iv=0;                       % Order of IV Polynomial = (bernO + iv)
    constrained = 1;            % Monotonicity Constraint (experience says you always want this on)
    xt = zeros(size(pt));       % No exogenous product characteristics
    trueS=0;                    % Evaluate at true market shares or not
    own = [1,1];                % [derivative of j, with respect to price of k]
    cross = [1,2];      
    
    % Estimate demand nonparametrically
        % If you want to include an additional covariate in all demand
        % functions, add an additional argument "marketvars" after included. If it is an
        % additional product characteristic, marketvars should be T x J
    [inv_sigma] = inverse_demand(s, pt, xt, zt, bernO, iv, J, constrained, included);
    
    % Calculate own- and cross- price elasticities at median xi, varying
        % price of first good from 25th to 75th percentile
    deltas = -1*median(pt).*ones(G,J);
    deltas(:,1) = -1*p_points;
    [esep_own(si,:), ~,stemp] = price_elasticity_priceIndex(inv_sigma, s,p_points, deltas, bernO, own, included,trueS,[]);
    [esep_cross(si,:), ~, ~] = price_elasticity_priceIndex(inv_sigma, s,p_points, deltas, bernO, cross, included,trueS,[]);
    
    % You can also calculate price elasticities at realized market shares
        % If calculating elasticities raises a warning, will be reported 
        % at the end of price_elasticity_priceIndex as a "bad market"
    trueS = 1;
    esep_own_dist(si,:) = price_elasticity_priceIndex(inv_sigma, s, pt, -1*pt, bernO, own, included,trueS,[]);
end



% ----------------------------------------------------------------
% Make figures
% ------------------------------------------------------

% Calculate true elasticities
inc = zeros(1,J);
inc(1) = 0.0001; % size of finite step
inc_cross = zeros(1,J);
inc_cross(2) = 0.0001;
p_grid = repmat(p_points,1,J); % grid of prices to evaluate elasticity 
p_grid(:,2:end) = median(pt(:,1));
esepTrue = zeros(G,1);
esepCrossTrue = zeros(G,1);
for i = 1:1:size(p_points)
   share_0 = logit_sharefun(beta, p_grid(i,:) , 0*ones(1,J),1);
   share_0c = logit_sharefun(beta, p_grid(i,:) , 0*ones(1,J),1);
   
   share_1 = logit_sharefun(beta, p_grid(i,:) + inc, 0*ones(1,J),1);
   share_1c = logit_sharefun(beta, p_grid(i,:) + inc_cross, 0*ones(1,J),1);
   esepTrue(i,1) =  (share_1 - share_0)/inc(1) * p_points(i,1)/share_0;
   esepCrossTrue(i,1) = (share_1c - share_0c)/inc(1) * p_grid(i,1)/share_0c;
end

% Figure 1: Own-price elasticity
figure(1)
clf
hold on
p0 = plot(p_points,esepTrue, 'k', 'LineWidth', 1.2);
p1_95 = plot(p_points, prctile(esep_own, 97.5), 'r--', 'LineWidth', 1.1); 
p1_5 = plot(p_points, prctile(esep_own, 2.5), 'r--', 'LineWidth', 1.1);
xlabel('Price')
ylabel('Own-Elasticity')
legend([p0 p1_95] ,'True' ,'95% CI')
hold off

% Figure 2: Cross-price elasticity
figure(2)
clf
hold on
p0 = plot(p_points,esepCrossTrue,'k', 'LineWidth', 1.2);
p1_95 = plot(p_points, prctile(esep_cross, 97.5), 'r--', 'LineWidth', 1.1); 
p1_5 = plot(p_points, prctile(esep_cross, 2.5), 'r--', 'LineWidth', 1.1);
xlabel('Price')
ylabel('Cross-Elasticity')
legend([p0 p1_95] ,'True' ,'95% CI')
hold off